package sun.security.smartcardio;

import java.util.Iterator;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Collections;
import javax.smartcardio.CardException;
import java.util.ArrayList;
import javax.smartcardio.CardTerminal;
import java.util.List;
import java.lang.ref.WeakReference;
import java.lang.ref.Reference;
import java.util.Map;
import javax.smartcardio.CardTerminals;

final class PCSCTerminals extends CardTerminals
{
    private static long contextId;
    private Map<String, PCSCTerminals.ReaderState> stateMap;
    private static final Map<String, Reference<TerminalImpl>> terminals;
    
    PCSCTerminals() {
    }
    
    static synchronized void initContext() throws PCSCException {
        if (PCSCTerminals.contextId == 0L) {
            PCSCTerminals.contextId = PCSC.SCardEstablishContext(0);
        }
    }
    
    private static synchronized TerminalImpl implGetTerminal(String s) {
        Reference<Object> reference = (Reference)PCSCTerminals.terminals.get(s);
        TerminalImpl terminalImpl = (reference != null) ? ((TerminalImpl)reference.get()) : null;
        if (terminalImpl != null) {
            return terminalImpl;
        }
        TerminalImpl referent = new TerminalImpl(PCSCTerminals.contextId, s);
        PCSCTerminals.terminals.put(s, new WeakReference(referent));
        return referent;
    }
    
    @Override
    public synchronized List<CardTerminal> list(CardTerminals.State obj) throws CardException {
        if (obj == null) {
            throw new NullPointerException();
        }
        try {
            String[] sCardListReaders = PCSC.SCardListReaders(PCSCTerminals.contextId);
            ArrayList list = new ArrayList(sCardListReaders.length);
            if (this.stateMap == null) {
                if (obj == CardTerminals.State.CARD_INSERTION) {
                    obj = CardTerminals.State.CARD_PRESENT;
                }
                else if (obj == CardTerminals.State.CARD_REMOVAL) {
                    obj = CardTerminals.State.CARD_ABSENT;
                }
            }
            for (String s : sCardListReaders) {
                TerminalImpl implGetTerminal = implGetTerminal(s);
                switch (obj) {
                    case ALL: {
                        list.add(implGetTerminal);
                        break;
                    }
                    case CARD_PRESENT: {
                        if (implGetTerminal.isCardPresent()) {
                            list.add(implGetTerminal);
                            break;
                        }
                        break;
                    }
                    case CARD_ABSENT: {
                        if (!implGetTerminal.isCardPresent()) {
                            list.add(implGetTerminal);
                            break;
                        }
                        break;
                    }
                    case CARD_INSERTION: {
                        PCSCTerminals.ReaderState readerState = (PCSCTerminals.ReaderState)this.stateMap.get(s);
                        if (readerState != null && readerState.isInsertion()) {
                            list.add(implGetTerminal);
                            break;
                        }
                        break;
                    }
                    case CARD_REMOVAL: {
                        PCSCTerminals.ReaderState readerState2 = (PCSCTerminals.ReaderState)this.stateMap.get(s);
                        if (readerState2 != null && readerState2.isRemoval()) {
                            list.add(implGetTerminal);
                            break;
                        }
                        break;
                    }
                    default: {
                        throw new CardException("Unknown state: " + obj);
                    }
                }
            }
            return Collections.unmodifiableList(list);
        }
        catch (PCSCException cause) {
            throw new CardException("list() failed", cause);
        }
    }
    
    @Override
    public synchronized boolean waitForChange(long lng) throws CardException {
        if (lng < 0L) {
            throw new IllegalArgumentException("Timeout must not be negative: " + lng);
        }
        if (this.stateMap == null) {
            this.stateMap = new HashMap();
            this.waitForChange(0L);
        }
        if (lng == 0L) {
            lng = -1L;
        }
        try {
            String[] sCardListReaders = PCSC.SCardListReaders(PCSCTerminals.contextId);
            int length = sCardListReaders.length;
            if (length == 0) {
                throw new IllegalStateException("No terminals available");
            }
            int[] array = new int[length];
            PCSCTerminals.ReaderState[] array2 = new PCSCTerminals.ReaderState[length];
            PCSCTerminals.ReaderState readerState;
            for (int i = 0; i < sCardListReaders.length; array2[i] = readerState, array[i] = readerState.get(), ++i) {
                readerState = (PCSCTerminals.ReaderState)this.stateMap.get(sCardListReaders[i]);
                if (readerState == null) {
                    readerState = new PCSCTerminals.ReaderState();
                }
            }
            int[] sCardGetStatusChange = PCSC.SCardGetStatusChange(PCSCTerminals.contextId, lng, array, sCardListReaders);
            this.stateMap.clear();
            for (int j = 0; j < length; ++j) {
                PCSCTerminals.ReaderState readerState2 = array2[j];
                readerState2.update(sCardGetStatusChange[j]);
                this.stateMap.put(sCardListReaders[j], readerState2);
            }
            return true;
        }
        catch (PCSCException cause) {
            if (cause.code == -2146435062) {
                return false;
            }
            throw new CardException("waitForChange() failed", cause);
        }
    }
    
    static List<CardTerminal> waitForCards(List<? extends CardTerminal> list, long n, boolean b) throws CardException {
        long n2;
        if (n == 0L) {
            n = -1L;
            n2 = -1L;
        }
        else {
            n2 = 0L;
        }
        String[] array = new String[list.size()];
        int n3 = 0;
        for (CardTerminal cardTerminal : list) {
            if (!(cardTerminal instanceof TerminalImpl)) {
                throw new IllegalArgumentException("Invalid terminal type: " + cardTerminal.getClass().getName());
            }
            array[n3++] = ((TerminalImpl)cardTerminal).name;
        }
        int[] sCardGetStatusChange = new int[array.length];
        Arrays.fill(sCardGetStatusChange, 0);
        while (true) {
            try {
                sCardGetStatusChange = PCSC.SCardGetStatusChange(PCSCTerminals.contextId, n2, sCardGetStatusChange, array);
                n2 = n;
                Object list2 = null;
                for (int i = 0; i < array.length; ++i) {
                    if ((sCardGetStatusChange[i] & 0x20) != 0x0 == b) {
                        if (list2 == null) {
                            list2 = new ArrayList();
                        }
                        list2.add(implGetTerminal(array[i]));
                    }
                }
                if (list2 == null) {
                    continue;
                }
                return Collections.unmodifiableList(list2);
            }
            catch (PCSCException cause) {
                if (cause.code == -2146435062) {
                    return Collections.emptyList();
                }
                throw new CardException("waitForCard() failed", cause);
            }
        }
    }
    
    static {
        terminals = new HashMap();
    }
    
    private static class ReaderState
    {
        private int current;
        private int previous;
        
        ReaderState() {
            this.current = 0;
            this.previous = 0;
        }
        
        int get() {
            return this.current;
        }
        
        void update(int current) {
            this.previous = this.current;
            this.current = current;
        }
        
        boolean isInsertion() {
            return !present(this.previous) && present(this.current);
        }
        
        boolean isRemoval() {
            return present(this.previous) && !present(this.current);
        }
        
        static boolean present(int n) {
            return (n & 0x20) != 0x0;
        }
    }
}
